package com.edgec.browserbackend.browser.service.Impl;

import com.edgec.browserbackend.account.domain.Account;
import com.edgec.browserbackend.account.exception.AccountErrorCode;
import com.edgec.browserbackend.account.repository.AccountRepository;
import com.edgec.browserbackend.browser.domain.LoginHistory;
import com.edgec.browserbackend.browser.domain.OperationHistory;
import com.edgec.browserbackend.browser.dto.*;
import com.edgec.browserbackend.browser.repository.LoginHistoryRepository;
import com.edgec.browserbackend.browser.repository.OperationHistoryRepository;
import com.edgec.browserbackend.browser.service.HistoryService;
import com.edgec.browserbackend.common.commons.error.ClientRequestException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;

import java.time.Instant;
import java.util.ArrayList;
import java.util.List;

@Service
public class HistoryServiceImpl implements HistoryService {

    @Autowired
    private AccountRepository accountRepository;

    @Autowired
    private LoginHistoryRepository loginHistoryRepository;

    @Autowired
    private OperationHistoryRepository operationHistoryRepository;


    @Override
    public void addLoginHistory(String username, LoginHistoryDto loginHistoryDto) {
        Account account = accountRepository.findById(username).orElse(null);
        if (account == null)
            throw new ClientRequestException(AccountErrorCode.NAMENOTEXIST);
        int num = loginHistoryRepository.countByAccount(username);
        if (num > 10000)
            loginHistoryRepository.delete(loginHistoryRepository.findFirstByAccount(username));
        LoginHistory loginHistory = new LoginHistory(loginHistoryDto);
        loginHistory.setLoginTime(Instant.now().toEpochMilli());
        loginHistory.setAccount(username);
        if (account.getParent() == null)
            loginHistory.setAdministrator(username);
        else
            loginHistory.setAdministrator(account.getParent());
        loginHistory.setNickname(account.getNickname());
        loginHistoryRepository.save(loginHistory);
    }

    @Override
    public void addOperationHistory(String username, OperationHistoryDto operationHistoryDto) {
        Account account = accountRepository.findById(username).orElse(null);
        if (account == null)
            throw new ClientRequestException(AccountErrorCode.NAMENOTEXIST);
        int num = operationHistoryRepository.countByAccount(username);
        if (num > 10000)
            operationHistoryRepository.delete(operationHistoryRepository.findFirstByAccount(username));
        OperationHistory operationHistory = new OperationHistory(operationHistoryDto);
        operationHistory.setOperationTime(Instant.now().toEpochMilli());
        operationHistory.setAccount(username);
        operationHistory.setNickname(account.getNickname());
        if (account.getParent() == null)
            operationHistory.setAdministrator(username);
        else
            operationHistory.setAdministrator(account.getParent());
        operationHistoryRepository.save(operationHistory);
    }

    @Override
    public HistoryPageResultDto getLoginHistories(String username, HistoryListRequestDto historyListRequestDto) {
        Account account = accountRepository.findById(username).orElse(null);
        if (account == null)
            throw new ClientRequestException(AccountErrorCode.NAMENOTEXIST);
        if (account.getPermission() < 8)
            throw new ClientRequestException(AccountErrorCode.NOPERMISSION);
        if (historyListRequestDto.getAmount() > 100)
            historyListRequestDto.setAmount(100);
        Pageable pageable = PageRequest.of(historyListRequestDto.getPage(), historyListRequestDto.getAmount());
        HistoryPageResultDto historyPageResultDto = new HistoryPageResultDto();
        Page<LoginHistory> loginHistoryPage;
        if (account.getParent() == null) {
            loginHistoryPage = loginHistoryRepository.findByAdministratorOrderByLoginTime(username, pageable);
        } else {
            loginHistoryPage = loginHistoryRepository.findByAccountOrderByLoginTime(username, pageable);
        }

        List<LoginHistoryDto> loginHistoryDtoList = new ArrayList<>();
        loginHistoryPage.getContent().forEach(x -> {
            loginHistoryDtoList.add(new LoginHistoryDto(x));
        });
        historyPageResultDto.setHistory(loginHistoryDtoList);
        PageInfo pageInfo = new PageInfo();
        pageInfo.setCurrentPage(loginHistoryPage.getNumber());
        pageInfo.setTotalPages(loginHistoryPage.getTotalPages());
        pageInfo.setTotalItems((int)loginHistoryPage.getTotalElements());
        historyPageResultDto.setHistoryPage(pageInfo);
        return historyPageResultDto;
    }

    @Override
    public HistoryPageResultDto getOperationHistories(String username, HistoryListRequestDto historyListRequestDto) {
        Account account = accountRepository.findById(username).orElse(null);
        if (account == null)
            throw new ClientRequestException(AccountErrorCode.NAMENOTEXIST);
        if (account.getPermission() < 8)
            throw new ClientRequestException(AccountErrorCode.NOPERMISSION);
        if (historyListRequestDto.getAmount() > 100)
            historyListRequestDto.setAmount(100);
        Pageable pageable = PageRequest.of(historyListRequestDto.getPage(), historyListRequestDto.getAmount());
        HistoryPageResultDto historyPageResultDto = new HistoryPageResultDto();
        Page<OperationHistory> operationHistories;
        if (account.getParent() == null) {
            operationHistories = operationHistoryRepository.findByAdministratorOrderByOperationTimeDesc(username, pageable);
        } else {
            operationHistories = operationHistoryRepository.findByAccountOrderByOperationTimeDesc(username, pageable);
        }

        List<OperationHistoryDto> operationHistoryDtos = new ArrayList<>();
        operationHistories.getContent().forEach(x -> {
            operationHistoryDtos.add(new OperationHistoryDto(x));
        });
        historyPageResultDto.setHistory(operationHistoryDtos);
        PageInfo pageInfo = new PageInfo();
        pageInfo.setCurrentPage(operationHistories.getNumber());
        pageInfo.setTotalPages(operationHistories.getTotalPages());
        pageInfo.setTotalItems((int)operationHistories.getTotalElements());
        historyPageResultDto.setHistoryPage(pageInfo);
        return historyPageResultDto;
    }
}
